# xDevs.com Plotting app for matplotlib and 3458A example
#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import csv
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import datetime
from scipy.ndimage.filters import gaussian_filter1d
#matplotlib.use('Agg')
    
fn_input = "log_10v_3458a_nplc200_feb2020.csv"
plot_out_filename = "log_10v_3458a_nplc200_feb2020.png"

temp = []
val = [[]]
ral = [[]]
polyz = []
for ival in range (0,17):
    val.append([])
for ival in range (0,17):
    ral.append([])
cnt = 0
alpha = []
beta = []
gain = []
tvalue = []
c1 = []
c2 = []
delta = []
zerotctemp = []
valzerotc = []
cnx = []
date = []
chop_samples = 25
num_lines = sum(1 for line in open(fn_input))  

for dval in range (0,num_lines-chop_samples):
    date.append(" ")
vref = [10.000000, 0, 0]

with open(fn_input) as csvfile:
    spamreader = csv.reader(csvfile, delimiter=';', quotechar='|')
    is_first = True
    dcount = 0
    for row in spamreader:
        cnt = cnt + 1
        dcount = dcount + 1
        if is_first:
            is_first = False
            continue
        #print ("%d" % cnt)
        if cnt <= chop_samples:
            continue
        else:
            cnx.append(cnt - (chop_samples+1))
            temp.extend([float(row[3])])               # Get TEMP readouts
            date[dcount - (chop_samples+1)] = datetime.datetime.strptime (row[0],"%d/%m/%Y-%H:%M:%S")
            #date.extend(row[0])#.strftime('%d/%m/%y-%h:%m:%s')
            ral[0].extend([  float(row[1]) ])                         # Read values from meter
            val[0].extend([ ( (float(row[1]) / vref[0]) - 1) * 1e6 ]) # Calculate deviation in ppm, relative to VREF value
csvfile.close()

tdelta = np.amax(temp) - np.amin(temp)
reftemp = 23.00
nominal = []
median = []

print ("Temperature min = %.3f , max = %.3f, delta = %.3f C" % (np.amin(temp), np.amax(temp), tdelta) )
print ("REF temp = %.3f C" % reftemp)
    
print ("\033[2J")                                           # Clear screen

for zval in range (0,1):                                  # Iterate labels, just one value here
    #nominal = [9.999957591e+00, 9.999963348e+00, 0, 0, 1.000001420e+01, 9.999958800e+00, 9.515830000e-01, 0, 0]
    nominal.append(np.mean(ral[zval][:30]))
    polyz.append(np.polyfit(temp, ral[zval], 2))
    c2.append(polyz[zval][0])
    c1.append(polyz[zval][1])
    gain.append(polyz[zval][2])
    tvalue.append(c2[zval] * reftemp*reftemp + c1[zval]*reftemp + gain[zval])
    alpha.append((c1[zval] + 2*reftemp*c2[zval]) / tvalue[zval]*1e6 )
    beta.append(c2[zval] / tvalue[zval]*1e6 )
    delta.append((tvalue[zval] - nominal[zval]) / nominal[zval] * 1e6)
    zerotctemp.append(-alpha[zval] / 2 / beta[zval] + reftemp)
    valzerotc.append(nominal[zval] + (zerotctemp[zval]) * c1[zval] + (zerotctemp[zval])*2*c2[zval])
    median.append(np.median(ral[zval]))
    
    if (gain[zval] != 0):
        print ("\033[37;1m\r\nNominal       :", nominal[zval])
        print ("Median        : %.8f" % median[zval])
        print ("Column number : %d " % zval), polyz[zval], "Total samples = %d" % len(ral[zval])
        print ("c1            : %e" % c1[zval])
        print ("c2            : %e" % c2[zval])
        print ("g             : %e" % gain[zval])
        print ("Value T23     : %.6f" % (tvalue[zval]))
        print ("\033[36;1mAlpha T23     : %.4f ppm/K       " % (alpha[zval])),
        print ("Beta      : %.4f ppm/K/2                " % (beta[zval])),
        print ("Delta T23 : %.3f ppm         \033[39;1m" % (delta[zval]))
        print ("\033[35;1mZero TC       : %.3f C \033[39;0m" % (zerotctemp[zval])),
        print ("\033[32;1m          Delta ZeroTC : %.6f\033[39;0m" % (valzerotc[zval]))
        
w = 20
h = 16
fig,ax = plt.subplots()
fig.patch.set_facecolor('#eeeeff')
fig.set_size_inches(w,h)
plt.tight_layout(rect=[0.07, 0.07, 0.94, 0.94], w_pad=0.4, h_pad=0.4)                                # Reduce margins around the graph
   
plt.xlabel('Time scale', fontsize=18)
plt.ylabel(('Deviation from %.8f, ppm' % vref[0]), fontsize=18) #$\mu$V

avgpower = 8
val1aver = gaussian_filter1d(val[0], sigma=avgpower)
temp2aver = gaussian_filter1d(temp, sigma=1)

colors = ['#e6194b', '#f58231', '#1f5119', '#3cb44b', '#008080', '#4363d8', '#000076', '#911eb4', '#f032e6', '#00a68c', '#664433', '#1f38e3', '#ff0000', '#ec18a0', '#dbb201', '#1c2b33', '#003355', '#fc007f']

avglinestype = 'solid'
avglinestypen = 'none'

plt.title('3458A 10V measurement example plot', fontsize=22)
ax.plot(date, val1aver, '-', label='CH1',     linewidth=3.0,             color=colors[0], linestyle=avglinestype, marker='.', ms=1)
ax.plot(date, val[0], '-'  , label='CH1 RAW', linewidth=1.5,  alpha=0.8, color=colors[1] )#, linestyle='none', marker='.', ms=6)
legendlabel = ["10V reference, filtered", "RAW data"]

start, end = ax.get_ylim()
#startx, endx = ax.get_xlim()
axis_y_step = 0.1
#axis_x_step = 1
ex = np.arange(1,2000)
#ax.yaxis.set_ticks(np.arange(start, end)) # axis_y_step))
ax.yaxis.set_ticks(np.arange(-3.3, -3.0, 0.01))         # Force Y-axis scale to specific values
#ax.xaxis.set_ticks(np.arange(17.5, 41.5, 0.5)) # axis_y_step))
#ax.xaxis.set_ticks(np.arange(1, 20000, 120)) # axis_y_step))
#ax.xaxis.set_ticks(np.arange(startx, endx)) #, axis_x_step))


ax2 = ax.twinx()  # instantiate a second axes that shares the same x-axis
ax2.set_ylabel('TEMP?', fontsize=18)  # we already handled the x-label with ax1
ax2.plot(date, temp2aver, '-.', label='DMM TEMP?', linewidth=3.0, color='#003300', linestyle='solid', marker='.', ms=1)
#ax2.tick_params(axis='y', labelcolor=color)
ax.legend(legendlabel, loc=4, fancybox=True, framealpha=1, borderpad=1, markerscale=12)
ax.grid(True, color='#cccccc')
ax2.yaxis.set_ticks(np.arange(34, 37, 0.1))            # Force second Y-axis to specific values
ax.xaxis.set_major_formatter(mdates.DateFormatter("%b %d, %H:%M:%S"))
ax.xaxis.set_minor_formatter(mdates.DateFormatter("%b %d, %H:%M:%S"))
ax2.tick_params(axis='both', which='major', labelsize=14)

vmed = []
vmin = []
vmax = []
sdev = []
tmin = np.amin(temp)
tmax = np.amax(temp)
temppp = tmax - 18
tempco = []

for label in (ax.get_xticklabels() + ax.get_yticklabels()):
    #label.set_fontname('Arial')
    label.set_fontsize(13)


for stats in range (0, 1):
    vmed.extend([0])
    vmin.extend([0])
    vmax.extend([0])
    sdev.extend([0])
    tempco.extend([0])
    
for stats in range (0, 1):
    vmed[stats] = np.median(val[stats])
    vmin[stats] = np.amin(val[stats])# - vstep
    vmax[stats] = np.amax(val[stats])# + vstep
    sdev[stats] = np.std(val[stats]) # SDEV
    tempco[stats] = ((vmax[stats] - vmin[stats]) / temppp) 
    if ((stats % 2) == 0):
        bgclr = "#f3f3ff"
    else:
        bgclr = "#ffffff"
        
    plt.text(0.01 + (stats * 0.062), 0.25, '%s' % (legendlabel[stats]),
        horizontalalignment='left',
        verticalalignment='top',backgroundcolor=bgclr,
        transform = ax.transAxes, size=18, color=colors[stats])
    plt.text(0.01 + (stats * 0.062), 0.21,'Sdev[%d] : %.3f ppm' % (stats, sdev[stats]),
        horizontalalignment='left',
        verticalalignment='top',backgroundcolor=bgclr,
        transform = ax.transAxes, size=12, color=colors[stats])
    plt.text(0.01 + (stats * 0.062), 0.18,'Median[%d] : %.3f ppm' % (stats, vmed[stats]),
        horizontalalignment='left',
        verticalalignment='top',backgroundcolor=bgclr,
        transform = ax.transAxes, size=12, color=colors[stats])
    plt.text(0.01 + (stats * 0.062), 0.15, 'Min[%d] : %.3f ppm' % (stats, vmin[stats]),
        horizontalalignment='left',
        verticalalignment='top', backgroundcolor=bgclr,
        transform = ax.transAxes, size=11, color=colors[stats])
    plt.text(0.01 + (stats * 0.062), 0.12, 'Max[%d] : %.3f ppm' % (stats, vmax[stats]),
        horizontalalignment='left',
        verticalalignment='top', backgroundcolor=bgclr,
        transform = ax.transAxes, size=11, color=colors[stats])
    plt.text(0.01 + (stats * 0.062), 0.09, 'Zero TC : %.3f C' % ( zerotctemp[stats] ),
        horizontalalignment='left',
        verticalalignment='top',backgroundcolor=bgclr,
        transform = ax.transAxes, size=12, color=colors[stats])
    plt.text(0.01 + (stats * 0.062), 0.06, 'Beta : %.3f ppm/K2' % ( beta[stats] ),
        horizontalalignment='left',
        verticalalignment='top',backgroundcolor=bgclr,
        transform = ax.transAxes, size=12, color=colors[stats])
    plt.text(0.01 + (stats * 0.062), 0.03, 'Alpha : %.3f ppm/K' % ( alpha[stats] ),
        horizontalalignment='left',
        verticalalignment='top',backgroundcolor=bgclr,
        transform = ax.transAxes, size=12, color=colors[stats])
ax.set_facecolor(bgclr)
    

smax = len(val[0])
plt.text(0.1, -0.07,'xDevs.com | https://xdevs.com/article/ks3458b',
    horizontalalignment='center',
    verticalalignment='center',
    transform = ax.transAxes, size=12)
plt.text(-0.04, -0.035,'Samples : %d' % smax,
    horizontalalignment='left',
    verticalalignment='top',
    transform = ax.transAxes, size=12)

#plt.ylim(-2,2)
fig.savefig(plot_out_filename, facecolor=fig.get_facecolor(), transparent=False)
plt.show()